#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

__all__ = ['BrowserApplet']

PROGNAME="setroubleshoot"
import gettext
gettext.bindtextdomain(PROGNAME, "/usr/share/locale")
gettext.textdomain(PROGNAME)
try:
    gettext.install(PROGNAME, "/usr/share/locale", 1)
except IOError:
    import __builtin__
    __builtin__.__dict__['_'] = unicode

# This import must come before importing gtk to silence warnings
from setroubleshoot.gui_utils import *

import os
import sys
import pygtk
pygtk.require("2.0")
import gtk, gtk.glade
import pango
import threading

import gobject 
import gnome.ui    

from setroubleshoot.analyze import *
from setroubleshoot.config import cfg
from setroubleshoot.errcode import *
from setroubleshoot.signature import *
from setroubleshoot.util import *
from setroubleshoot.log import *
from setroubleshoot.rpc import *
from setroubleshoot.rpc_interfaces import *
import gtkhtml2

#------------------------------------------------------------------------------

NUM_COLUMNS = 6
(PYOBJECT_COLUMN, FILTER_TYPE_COLUMN, DATE_COLUMN, COUNT_COLUMN, CATEGORY_COLUMN,
 SUMMARY_COLUMN) = range(NUM_COLUMNS)

seen_text_attributes = pango.AttrList()

not_seen_text_attributes = pango.AttrList()
not_seen_text_attributes.insert(pango.AttrWeight(pango.WEIGHT_HEAVY, 0, -1))

#------------------------------------------------------------------------------

class LogfileThread(threading.Thread,
                    RpcManage,
                    SETroubleshootDatabaseNotifyInterface,
                    gobject.GObject):

    __gsignals__ = {
        'database_bind': 
        (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT)),
        'signatures_updated': 
        (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT)),
        }


    def __init__(self, browser, logfile_path):
        # parent class constructors
        threading.Thread.__init__(self)
        gobject.GObject.__init__(self)
        RpcManage.__init__(self)
        self.connect_rpc_interface('SETroubleshootDatabaseNotify', self)
        self.browser = browser
        self.logfile_path = logfile_path
        self.analyzer = LogfileAnalyzer(self.logfile_path, notify=self.browser)
        self.database = self.analyzer.database_rpc

    def run(self):
        self.process_logfile()

    def progress_callback(self, progress):
        self.browser.statusbar_widget.progress.set_fraction(progress)

    def process_logfile(self):
        if debug:
            log_gui.debug("on_open_logfile: self.logfile_path=%s", self.logfile_path)

        self.browser.signal_connection_database_bind.connect(self, self.browser.on_logfile_database_bind)
        self.emit('database_bind', self.database, self.analyzer.database.properties)
        self.browser.update_visit('logfile')

        self.browser.set_status_message('opening %s' % self.logfile_path)
        self.browser.statusbar_widget.progress.set_fraction(0.0)
        self.browser.statusbar_widget.progress.set_text(self.browser.logfile_database_properties.friendly_name)
        # FIXME: instead of load_data() it should clear the model, set initial selection
        self.browser.load_data()
        self.analyzer.analyze_logfile(self.progress_callback)

    def signatures_updated(self, type, item):
        if debug:
            log_rpc.debug('signatures_updated() LogfileThread: type=%s item=%s', type, item)
        self.emit('signatures_updated', type, item)
        
gobject.type_register(LogfileThread)

#------------------------------------------------------------------------------

class BrowserStatusBar(gtk.HBox):
    # These should sum to 1.0
    visit_msg_proportion  = 0.2
    status_msg_proportion = 0.6
    progress_proportion   = 0.2

    def __init__(self):
        gtk.HBox.__init__(self)
        self.connect('size-allocate', self.on_size_allocate)

        self.connect_icon = gtk.Image()
        self.connect_icon.set_from_stock(gtk.STOCK_DISCONNECT, gtk.ICON_SIZE_SMALL_TOOLBAR)
        self.connect_icon.show()
    
        self.connect_icon_frame = gtk.Frame()
        self.connect_icon_frame.add(self.connect_icon)
        self.pack_start(self.connect_icon_frame, False, False, 0)
        self.connect_icon_frame.show()
	
        self.visit_msg = gtk.Label("visiting")
        self.visit_msg.set_ellipsize(pango.ELLIPSIZE_END)
        self.visit_msg.set_alignment(0.0, 0.5)
        self.visit_msg_frame = gtk.Frame()
        self.visit_msg_frame.add(self.visit_msg)
        self.pack_start(self.visit_msg_frame, False, False, 0)
        self.visit_msg_frame.show()
        self.visit_msg.show()
	
        self.status_msg = gtk.Label("status")
        self.status_msg.set_ellipsize(pango.ELLIPSIZE_END)
        self.status_msg.set_alignment(0.0, 0.5)
        self.status_msg_frame = gtk.Frame()
        self.status_msg_frame.add(self.status_msg)
        self.pack_start(self.status_msg_frame, False, False, 0)
        self.status_msg_frame.show()
        self.status_msg.show()
	
        self.progress = gtk.ProgressBar()
        self.progress.set_ellipsize(pango.ELLIPSIZE_END)
        self.progress_frame = gtk.Frame()
        self.progress_frame.add(self.progress)
        self.pack_start(self.progress_frame, False, False, 0)
        self.progress_frame.show()
        self.progress.show()
	
        self.show()
        
    def on_size_allocate(self, widget, allocation):
        max_x = allocation.x + allocation.width
        
        child_rect = gtk.gdk.Rectangle(allocation.x, allocation.y,
                                 allocation.width, allocation.height)

        # Compute size for connection icon
        icon_width, icon_height = self.connect_icon_frame.size_request()

        # Total space available for fixed sized items
        fixed_width_total = icon_width
        fixed_width_total = min(allocation.width, fixed_width_total)
        fixed_width_remaining = fixed_width_total

        # Total space available remaining for scaled items
        scaled_width_total = allocation.width - fixed_width_total
        scaled_width_remaining = scaled_width_total

        # Move left to right laying out items:
        # [connection icon][visit msg][status msg][progress bar]

        # Connection Icon
        child_rect.width = min(fixed_width_total, icon_width)
        self.connect_icon_frame.size_allocate(child_rect)
        child_rect.x += child_rect.width
        fixed_width_remaining = fixed_width_remaining - child_rect.width

        # Visit Message
        child_rect.width = int(scaled_width_total * self.visit_msg_proportion)
        if (child_rect.x + child_rect.width) > max_x:
            child_rect.width = max_x - child_rect.x
        self.visit_msg_frame.size_allocate(child_rect)
        scaled_width_remaining = scaled_width_remaining - child_rect.width
        child_rect.x += child_rect.width

        # Status Message
        child_rect.width = int(scaled_width_total * self.status_msg_proportion)
        if (child_rect.x + child_rect.width) > max_x:
            child_rect.width = max_x - child_rect.x
        self.status_msg_frame.size_allocate(child_rect)
        scaled_width_remaining = scaled_width_remaining - child_rect.width
        child_rect.x += child_rect.width

        # Progress Bar
        # Note, we don't compute the proportionate space for the last scaled item
        # because of potential rounding errors, we just use whats left
        child_rect.width = scaled_width_remaining
        if child_rect.x + child_rect.width > max_x:
            child_rect.width = max(0, max_x - child_rect.x)
        self.progress_frame.size_allocate(child_rect)

        return True

#------------------------------------------------------------------------------

class SignalConnection(object):
    def __init__(self, signal, emitter=None, id=None):
        self.signal = signal
        self.emitter = emitter
        self.id = id

    def connect(self, emitter, handler):
        self.disconnect()
        self.emitter = emitter
        if self.emitter is not None:
            self.id = self.emitter.connect(self.signal, handler)
        

    def disconnect(self):
        if self.emitter is not None and self.id is not None:
            self.emitter.disconnect(self.id)
        self.emitter = None
        self.id = None

#------------------------------------------------------------------------------

class BrowserApplet(Applet):

    # --- Initialization ---

    def __init__(self, username, server):
        self.username = username
        self.server = server
        self.cur_siginfo = None
        self.alert_display = None
        self.window_delete_hides = True
        self.load_in_progress = False
        self.audit_database = None
        self.audit_database_properties = None
        self.logfile_database = None
        self.logfile_database_properties = None
        self.visit_target = None
        self.logfile_thread = None

        # How long an alert must be displayed before it's marked as having been seen
        self.seen_after_displayed_seconds = 3

        self.date_format = '%a %b %e %Y %l:%M %p'

        program_name = _('setroubleshoot browser')
        program_version = '1.0'
        gnome.init(program_name, program_version)
        self.window_name = _("SETroubleshoot Browser")
        gtk.glade.set_custom_handler(self.glade_custom_handler)
        glade_file = cfg.get('alert','browser_glade_file')
        self.wTree = gtk.glade.XML(glade_file)
        self.init_widgets()

        self.hide()

        self.update_connection_state(server.get_connection_state())

        self.signal_connection_connection_state_change  = SignalConnection('connection_state_change')
        self.signal_connection_connection_pending_retry = SignalConnection('connection_pending_retry')
        self.signal_connection_signatures_updated       = SignalConnection('signatures_updated')
        self.signal_connection_database_bind            = SignalConnection('database_bind')

        self.signal_connection_database_bind.connect(self.server, self.on_audit_database_bind)

        self.update_visit('audit')
        
        dic = {'on_browser_delete_event'       : self.on_close,
               'on_open_logfile_activate'      : self.on_open_logfile,
               'on_close_activate'             : self.on_close,
               'on_print_activate'             : self.on_print,
               'on_delete_activate'            : self.on_delete,
               'on_save_as_activate'           : self.on_save_as,
               'on_view_audit_activate'        : self.on_view_audit,
               'on_view_logfile_activate'      : self.on_view_logfile,
               'on_report_bug_activate'        : self.on_report_bug,
               'on_about_activate'             : self.on_about,
               } 
        self.wTree.signal_autoconnect(dic) 

    def glade_custom_handler (self, glade, function_name, widget_name, str1, str2, int1 , int2):
        handler = getattr(self, function_name)
        if handler is None:
            return None
	return handler(widget_name, str1, str2, int1, int2)

    def create_statusbar(self, widget_name, str1, str2, int1 , int2):
        widget = BrowserStatusBar()
        return widget

    def on_realize(self, widget):
        # position the divider between the top and bottom pane
        vpane_rect = self.vpane_widget.get_allocation()
        self.vpane_widget.set_position(vpane_rect.height/4)

    def init_widgets(self):
        self.get_widget('browser')
        self.browser_widget.connect_after("realize", self.on_realize)

        self.get_widget("vpane")
        self.get_widget("statusbar")

        self.init_list_view('alert_list',
             [ColumnAttr(PYOBJECT_COLUMN,    None,          False, 'pyobject'),
              ColumnAttr(FILTER_TYPE_COLUMN, _('Filter'),   True,  'toggle',   sort_id=FILTER_TYPE_COLUMN, cell_data_func=self.filter_type_cell_data, edit_callback_func=self.on_filter_toggle),
              ColumnAttr(COUNT_COLUMN,       _('Count'),    True,  'int',      sort_id=COUNT_COLUMN,       cell_data_func=self.count_cell_data),
              ColumnAttr(DATE_COLUMN,        _('Date'),     True,  'pyobject', sort_id=DATE_COLUMN,        cell_data_func=self.date_cell_data),
              ColumnAttr(CATEGORY_COLUMN,    _('Category'), True,  'string',   sort_id=CATEGORY_COLUMN,    cell_data_func=self.text_cell_data),
              ColumnAttr(SUMMARY_COLUMN,     _('Summary'),  True,  'string',                               cell_data_func=self.text_cell_data)])

                             
        # alternate row color for easy reading
        self.alert_list_widget.set_rules_hint(True)

        # only permit single selections
        self.alert_list_selection = self.alert_list_widget.get_selection()
        self.alert_list_selection.set_mode(gtk.SELECTION_SINGLE)
        # connect to the selection signal, can't be done with autoconnect
        self.alert_list_selection.connect('changed', self.on_selection_changed)
        # initially select first row
        self.alert_list_selection.select_path((0))

        # FIXME: cruft
        #self.alert_list_widget.connect('button-press-event', self.on_alert_list_button_press)

        self.alert_list_model.connect('sort-column-changed', self.on_sort_column_changed)
        self.alert_list_model.set_sort_func(DATE_COLUMN, self.sort_date_column, DATE_COLUMN)

        # set initial sort order
        self.sort_column_id = DATE_COLUMN
        self.sort_order = gtk.SORT_DESCENDING
        self.alert_list_model.set_sort_column_id(self.sort_column_id, self.sort_order)

        # description
        self.get_widget("alert_detail_scrolled_window")
        view, doc = self.create_htmlview(self.alert_detail_scrolled_window_widget)
        doc.connect("link-clicked", self.link_clicked)
        self.description_doc = doc
        self.description_view = view

    def on_report_bug(self, widget):
        # FIXME - Should be specific to the alert
        os.spawnl(os.P_NOWAIT, "/usr/bin/htmlview","htmlview", "http://bugzilla.redhat.com/bugzilla/enter_bug.cgi")
        return True

    def link_clicked(self, arg1, arg2):
        #print "htmlview", "htmlview %s" % arg2
        os.spawnl(os.P_NOWAIT, "/usr/bin/htmlview","htmlview", arg2)
        
    def present(self):
        self.browser_widget.present()

    # --- Column Handling ---

    def sort_date_column(self, model, iter1, iter2, column_index):
        timestamp1 = model.get_value(iter1, column_index)
        timestamp2 = model.get_value(iter2, column_index)
        if timestamp1 is None or timestamp2 is None:
            return 0
        return timestamp1.cmp(timestamp2)

    def count_cell_data(self, column, cell, model, iter, column_index):
        siginfo = model.get_value(iter, PYOBJECT_COLUMN)
        user_data = siginfo.get_user_data(self.username)
        text = model.get_value(iter, column_index)
        if user_data.seen_flag:
            cell.set_property("attributes", seen_text_attributes)
        else:
            cell.set_property("attributes", not_seen_text_attributes)

        # FIXME: this should be a global property, not set each time
        cell.set_property("xalign", 1.0)

        cell.set_property('text', text) 


        return

    def date_cell_data(self, column, cell, model, iter, column_index):
        siginfo = model.get_value(iter, PYOBJECT_COLUMN)
        user_data = siginfo.get_user_data(self.username)
        if user_data.seen_flag:
            cell.set_property("attributes", seen_text_attributes)
        else:
            cell.set_property("attributes", not_seen_text_attributes)

        timestamp = model.get_value(iter, column_index)
        if timestamp is None:
            return
        cell.set_property('text', timestamp.format(self.date_format))
        return

    def text_cell_data(self, column, cell, model, iter, column_index):
        siginfo = model.get_value(iter, PYOBJECT_COLUMN)
        user_data = siginfo.get_user_data(self.username)
        text = model.get_value(iter, column_index)
        if user_data.seen_flag:
            cell.set_property("attributes", seen_text_attributes)
        else:
            cell.set_property("attributes", not_seen_text_attributes)
        return

    def filter_type_cell_data(self, column, cell, model, iter, column_index):
        siginfo = model.get_value(iter, PYOBJECT_COLUMN)
        user_data = siginfo.get_user_data(self.username)

        if user_data.filter.filter_type == FILTER_NEVER:
            cell.set_property('active', False)
        else:
            cell.set_property('active', True)
        return

    # --- Utilities ---

    def mark_seen(self, database, siginfo):
        if self.cur_siginfo == siginfo:
            user_data = siginfo.get_user_data(self.username)
            user_data.seen_flag = True
            self.alert_list_widget.queue_draw()
            database.set_user_data(siginfo.sig, self.username, 'seen_flag', True)
        return False

    def get_selected_iter(self):
        model, iter = self.alert_list_selection.get_selected()
        return iter

    def get_selected_row(self):
        model, iter = self.alert_list_selection.get_selected()
        if iter is None:
            return None
        path = model.get_path(iter)
        return path[0]

    def get_row_from_iter(self, iter):
        path = self.alert_list_model.get_path(iter)
        return path[0]

    def get_selected_siginfo(self):
        model, iter = self.alert_list_selection.get_selected()
        if iter is None:
            return None
        siginfo = model.get_value(iter, PYOBJECT_COLUMN)
        return siginfo

    def get_siginfo_at_row(self, row):
        iter = self.alert_list_model.get_iter((row))
        if iter is None:
            return None
        siginfo = self.alert_list_model.get_value(iter, PYOBJECT_COLUMN)
        return siginfo

    def get_siginfo_at_iter(self, iter):
        if iter is None:
            return None
        siginfo = self.alert_list_model.get_value(iter, PYOBJECT_COLUMN)
        return siginfo

    # --- View Management ---

    def show(self):
        self.browser_widget.show()

    def hide(self):
        self.browser_widget.hide()

    def __get_table_color(self):
        style = self.browser_widget.get_style()
        c = style.bg[gtk.STATE_NORMAL]
        color = "#%.2X%.2X%.2X" % (c.red / 256, c.green / 256, c.blue / 256)
        return color

    def update_alert_view(self, siginfo):
        self.cur_siginfo = siginfo
        self.description_doc.clear()
        self.description_doc.open_stream("text/html")
        html = (siginfo.format_html(self.__get_table_color()))
        if debug:
            #log_gui.debug(html)
            pass
        self.description_doc.write_stream(html)
        self.description_doc.close_stream()

        self.alert_detail_scrolled_window_widget.show_all()
        
        gobject.timeout_add(1000*self.seen_after_displayed_seconds, self.mark_seen, self.get_visit_database(), siginfo)

    def clear_alert_view(self):
        self.description_doc.clear()
        self.alert_detail_scrolled_window_widget.queue_draw()
        #self.alert_detail_scrolled_window_widget.show_all()

    def init_gui_data(self):
        self.alert_list_model.clear()
        count = 0
        for siginfo in self.sigs.signature_list:
            count += 1
            self.new_model_row(siginfo)
        # sort the new data
        self.alert_list_model.set_sort_column_id(self.sort_column_id, self.sort_order)

        # initially select first row
        self.alert_list_selection.select_path((0))
        self.settings = None

    def find_local_id_in_model(self, local_id):
        iter = self.alert_list_model.get_iter_first()
        while iter:
            row_siginfo = self.alert_list_model.get_value(iter, PYOBJECT_COLUMN)
            if row_siginfo.local_id == local_id:
                return iter
            iter = self.alert_list_model.iter_next(iter)

        return None

    def insert_siginfo_into_model(self, siginfo, sort=False):
        iter = self.find_local_id_in_model(siginfo.local_id)
        if iter is None:
            if debug:
                log_gui.debug("insert_siginfo_into_model(): new")
            iter = self.new_model_row(siginfo)
        else:
            if debug:
                log_gui.debug("insert_siginfo_into_model(): replace")
            self.update_model_row(iter, siginfo)
        if sort:
            self.alert_list_model.set_sort_column_id(self.sort_column_id, self.sort_order)
            

    def update_model_row(self, iter, siginfo):
        count = siginfo.report_count
        date = siginfo.last_seen_date
        category = get_text_default(siginfo.category)
        summary = remove_linebreaks(siginfo.solution.summary)

        self.alert_list_model.set(iter, PYOBJECT_COLUMN, siginfo)
        self.alert_list_model.set(iter, COUNT_COLUMN,    count)
        self.alert_list_model.set(iter, DATE_COLUMN,     date)
        self.alert_list_model.set(iter, CATEGORY_COLUMN, category)
        self.alert_list_model.set(iter, SUMMARY_COLUMN,  summary)

    def new_model_row(self, siginfo):
        iter = self.alert_list_model.append()
        self.update_model_row(iter, siginfo)
        return iter

    def get_visit_database(self):
        database = None
        if self.visit_target == 'audit':
            database = self.audit_database
        elif self.visit_target == 'logfile':
            database = self.logfile_database
        return database

    def update_visit(self, target):
        if self.visit_target == target:
            #return
            pass

        self.visit_target = target

        self.clear_visit_message()
        if target == 'audit':
            self.signal_connection_connection_state_change.connect(self.server, self.on_connection_state_change)
            self.signal_connection_connection_pending_retry.connect(self.server, self.on_connection_pending_retry)
            self.signal_connection_signatures_updated.connect(self.server, self.on_signatures_updated)
            if self.audit_database_properties is not None:
                self.set_visit_message(self.audit_database_properties.friendly_name)
            else:
                self.set_visit_message(_('Unconnected Audit'))
        
        elif target == 'logfile':
            self.signal_connection_connection_state_change.connect(self.server, self.on_connection_state_change)
            self.signal_connection_connection_pending_retry.disconnect()
            if self.logfile_database_properties is not None:
                self.signal_connection_signatures_updated.connect(self.logfile_thread, self.on_signatures_updated)
                self.set_visit_message(self.logfile_database_properties.friendly_name)
            else:
                self.signal_connection_signatures_updated.disconnect()
                self.set_visit_message(_('Unspecified logfile'))

        self.load_data()

    # --- Data Handling ---

    def progress_pulse(self):
        if self.load_in_progress:
            self.statusbar_widget.progress.pulse()
            return True
        else:
            return False

    def on_audit_database_bind(self, emitter, database, properties):
        self.audit_database = database
        self.audit_database_properties = properties
        if self.visit_target == 'audit':
            self.set_visit_message(self.audit_database_properties.friendly_name)
            self.load_data()

    def on_logfile_database_bind(self, emitter, database, properties):
        self.logfile_database = database
        self.logfile_database_properties = properties
        if self.visit_target == 'logfile':
            self.set_visit_message(self.logfile_database_properties.friendly_name)

    def signatures_updated(self, type, item):
        print "signatures_updated() browser: type=%s, item=%s" % (type, item)
        if debug:
            log_gui.debug("signatures_updated() browser: type=%s, item=%s", type, item)

        def new_siginfo_callback(sigs):
            if debug:
                log_gui.debug("new_siginfo_callback(), type=%s %s", type, str(sigs))

            for siginfo in sigs.signature_list:
                self.insert_siginfo_into_model(siginfo, True)

        database = self.get_visit_database()

        if type == 'add' or type == 'modify':
            async_rpc = database.query_alerts(item)
            async_rpc.add_callback(new_siginfo_callback)
        elif type == 'delete':
            iter = self.find_local_id_in_model(item)
            if iter is not None:
                if self.get_row_from_iter(iter) == self.get_selected_row():
                    new_selected_row = min(self.get_selected_row(), len(self.alert_list_model)-2)
                    print "new_selected_row=%s" % new_selected_row
                    if new_selected_row < 0:
                        new_selected_row = None
                else:
                    new_selected_row = None
                self.alert_list_model.remove(iter)
                if new_selected_row is not None:
                    self.alert_list_selection.select_path((new_selected_row))
        else:
            raise ProgramError(ERR_UNKNOWN_VALUE, "signatures_updated: type = %s not recognized" % type)
            
    def on_signatures_updated(self, alert_client, type, item):
        self.signatures_updated(type, item)

    def validate_data(self):
        timestamp = TimeStamp()
        for siginfo in self.sigs.signature_list:
            if siginfo.first_seen_date is None:
                siginfo.first_seen_date = timestamp
            if siginfo.last_seen_date is None:
                siginfo.last_seen_date = timestamp

    def load_data(self):
        self.clear_data()

        database = self.get_visit_database()
        if debug:
            log_gui.debug("load_data(): visit_target=%s database=%s", self.visit_target, database)
        if database is not None:
            criteria = '*'
            self.set_status_message(_('loading data ...'))
            #self.statusbar_widget.progress.set_fraction(.5)
            self.load_in_progress = True
            self.statusbar_widget.progress.pulse()
            gobject.timeout_add(250, self.progress_pulse)
            self.statusbar_widget.progress.set_text(_("Server Load"))
            async_rpc = database.query_alerts(criteria)
            async_rpc.add_callback(self.query_alerts_callback)
            async_rpc.add_errback(self.query_alerts_error)


    def query_alerts_callback(self, sigs):
        if debug:
            log_gui.debug("query_alerts_callback():")
        self.sigs = sigs
        self.validate_data()
        self.init_gui_data()
        self.set_status_message(_('loading data done'), 15)
        self.load_in_progress = False
        self.statusbar_widget.progress.set_text("")
        self.statusbar_widget.progress.set_fraction(0)


    def query_alerts_error(self, method, errno, strerror):
        log_gui.error("query_alerts: [%d] %s", errno, strerror)
        self.set_status_message("_(load data failed: %s", strerror)
        self.load_in_progress = False
        self.statusbar_widget.progress.set_text("")
        self.statusbar_widget.progress.set_fraction(0)


    def clear_data(self):
        self.sigs = SEFaultSignatureSet()
        self.init_gui_data()
        self.clear_alert_view()

    # --- Event Handlers ---

    def on_close(self, *args):
        if self.window_delete_hides:
            self.hide()
            return True
        else:
            gtk.main_quit()
            

    def on_connection_state_change(self, alert_client, connection_state):

        if debug:
            log_gui.debug("on_connection_state_change(): state=%s", connection_state)

        self.update_connection_state(connection_state)

    def on_connection_pending_retry(self, alert_client, seconds_pending):
        if seconds_pending <= 0:
            self.set_connection_status(False, _('retrying connection'))
        else:
            minutes = seconds_pending / 60
            seconds = seconds_pending % 60
            if minutes:
                pending = _("%d minutes %d seconds") % (minutes, seconds)
            else:
                pending = _("%d seconds") % (seconds)
                self.set_connection_status(False, _('retrying connection in %s' % pending))
            

    def on_about(self, widget):
        dlg = self.wTree.get_widget("aboutdialog")
        dlg.run()
        dlg.hide()

    def on_save_as(self, widget):
        dialog = gtk.FileChooserDialog("Save..",
                                       None,
                                       gtk.FILE_CHOOSER_ACTION_SAVE,
                                       (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                        gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        dialog.set_default_response(gtk.RESPONSE_OK)

        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*")
        dialog.add_filter(filter)

        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            siginfo = self.get_selected_siginfo()
            if siginfo is not None:
                self.save(dialog.get_filename(), siginfo)
        dialog.destroy()

    def save(self, file, siginfo):
        fd=open(file, "w")
        fd.write(siginfo.format_text())
        fd.close()
        
    def on_open_logfile(self, widget):
        title = _("Open logfile for scanning")
        options = ''
        dialog = FileChooserDialog()
        filename = dialog.get_filename(title)
        if filename is None:
            return
        if debug:
            log_gui.debug("on_open_logfile: filename=%s", filename)

        try:
            self.logfile_thread = LogfileThread(self, filename)
            self.logfile_thread.run()
        except ProgramError, e:
            self.display_error(e.strerror)

    def draw_page(self, operation, context, page_nr):
        siginfo = self.get_selected_siginfo()
        if siginfo is not None:
            siginfo = model.get_value(iter, PYOBJECT_COLUMN)
            user_data = siginfo.format_text()
        else:
            return
        cr = context.get_cairo_context()
        layout = context.create_pango_layout()
        layout.set_text(user_data)
        cr.show_layout(layout)

    def on_print(self, widget):
        dialog = gtk.PrintOperation()
        if self.settings != None:
            dialog.set_print_settings(self.settings)
        dialog.connect("draw_page", self.draw_page)
        dialog.set_n_pages(1)

        res = dialog.run(gtk.PRINT_OPERATION_ACTION_PRINT_DIALOG)

        if res == gtk.PRINT_OPERATION_RESULT_APPLY:
            self.settings = dialog.get_print_settings()
      
    def on_view_audit(self, widget):
        if debug:
            log_gui.debug("on_view_audit()")
        self.update_visit('audit')

    def on_view_logfile(self, widget):
        if debug:
            log_gui.debug("on_view_logfile()")
        self.update_visit('logfile')

    def on_selection_changed(self, selection):
        siginfo = self.get_selected_siginfo()
        if siginfo:
            self.update_alert_view(siginfo)
        else:
            self.clear_alert_view()

    def on_sort_column_changed(self, treesortable):
        tv_column = self.alert_list_widget.get_column(self.sort_column_id)
        tv_column.set_sort_indicator(False)

        self.sort_column_id, self.sort_order = treesortable.get_sort_column_id()
        tv_column = self.alert_list_widget.get_column(self.sort_column_id)
        tv_column.set_sort_indicator(True)

    def on_filter_toggle(self, cell, path, model):
        iter = model.get_iter_from_string(path)
        siginfo = model.get_value(iter, PYOBJECT_COLUMN)
        user_data = siginfo.get_user_data(self.username)

        if user_data.filter.filter_type == FILTER_NEVER:
            user_data.filter.filter_type = FILTER_ALWAYS
        else:
            user_data.filter.filter_type = FILTER_NEVER
        self.server.set_filter(siginfo.sig, self.username,
                               user_data.filter.filter_type, '')


    def verify(self, message):
        dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, message)
        dlg.set_position(gtk.WIN_POS_MOUSE)
        dlg.show_all()
        rc = dlg.run()
        dlg.destroy()
        return rc
    
    def display_error(self, message):
        self.set_status_message(message)
        dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, message)
        dlg.set_position(gtk.WIN_POS_CENTER)
        dlg.show_all()
        rc = dlg.run()
        dlg.destroy()
        return rc
    
    def on_delete(self, widget):
        siginfo = self.get_selected_siginfo()
        if siginfo:
            if self.verify(_("Are you sure you want to delete current alert?")) == gtk.RESPONSE_YES:
                self.server.delete_signature(siginfo.sig)

        return True            # return True ==> handled, False ==> propagate

    def update_connection_state(self, connection_state):

        if debug:
            log_gui.debug("on_connection_state_change(): state=%s", connection_state)

        self.connection_state = connection_state

        if connection_state == 'closed':
            self.clear_data()
            self.set_connection_status(False, _('connection closed'))
        elif connection_state == 'open':
            self.set_connection_status(True, _('connected'))
        elif connection_state == 'retry':
            self.clear_data()
        elif connection_state == 'hup':
            self.set_connection_status(False, _('connection lost'))
        elif connection_state == 'error':
            self.set_connection_status(False, _('connection failed'))
        elif connection_state == 'invalid':
            pass
        elif connection_state == 'timeout':
            self.set_connection_status(False, _('connection timed out'))
        else:
            log_gui.error("on_connection_state_change: unknown state (%s)", state)


    def set_status_message(self, msg, timeout=None):
        self.status_message = msg
        self.statusbar_widget.status_msg.set_text(self.status_message)
        if timeout is not None:
            def clear_msg():
                if self.status_message == msg:
                    self.clear_status_message()
            gobject.timeout_add(timeout*1000, clear_msg)
            

    def clear_status_message(self):
        self.status_message = ''
        self.statusbar_widget.status_msg.set_text(self.status_message)

    def set_visit_message(self, msg):
        self.visit_message = msg
        self.statusbar_widget.visit_msg.set_text(msg)

    def clear_visit_message(self):
        self.statusbar_widget.visit_msg.set_text('')

    def set_connection_status(self, is_connected, msg=None):
        if is_connected:
            self.statusbar_widget.connect_icon.set_from_stock(gtk.STOCK_CONNECT, gtk.ICON_SIZE_SMALL_TOOLBAR)
        else:
            self.statusbar_widget.connect_icon.set_from_stock(gtk.STOCK_DISCONNECT, gtk.ICON_SIZE_SMALL_TOOLBAR)
            self.clear_visit_message()
            
        if msg is not None:
            self.set_status_message(msg, timeout=15)


#-----------------------------------------------------------------------------

# -- Main --

if __name__ == "__main__":
    import getopt

    def usage():
        print '''
        -h help
'''

    try:
        opts, args = getopt.getopt(sys.argv[1:], "h", ["help"])
    except getopt.GetoptError:
        # print help information and exit:
        usage()
        sys.exit(2)

    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit()

    browser_applet = BrowserApplet()
    browser_applet.show()

    gtk.main()
