#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

__all__ = [
          ]

from setroubleshoot.config import cfg
from setroubleshoot.client_server import *
from setroubleshoot.util import *
from setroubleshoot.log import *

import threading
import subprocess
from select import select

#------------------------------------------------------------------------------

STDOUT = 1
STDERR = 2
STATE  = 3



class CmdJob(threading.Thread):
    # class variables
    bufsize = 4096

    def __init__(self, job_id, cmd, callback):
        # parent class constructor
        threading.Thread.__init__(self)
        self.job_id = job_id
        self.cmd = cmd
        self.callback = callback
        self.sub_process = None

    def run(self):
        self.sub_process = subprocess.Popen(self.cmd, \
            stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE, \
            close_fds=True, shell=True)
        self.stdout = self.sub_process.stdout
        self.stderr = self.sub_process.stderr

        read_watch = [self.stdout, self.stderr]
        while read_watch:
            readable = select(read_watch, [], [])[0]
            if readable:
                for fd in readable:
                    if fd == self.stdout:
                        data = fd.read(CmdJob.bufsize)
                        if not data:
                            read_watch.remove(fd)
                        else:
                            self.callback(self.job_id, STDOUT, data)
                    if fd == self.stderr:
                        data = fd.read(CmdJob.bufsize)
                        if not data:
                            read_watch.remove(fd)
                        else:
                            self.callback(self.job_id, STDERR, data)

        self.returncode = self.sub_process.wait()
        self.callback(self.job_id, STATE, self.returncode)
        
#------------------------------------------------------------------------------
class CmdJobPool:

    def __init__(self):
        self.job_id = 1
        self.jobs = {}

    def new_job(self, cmd, callback):
        job_id = self.job_id
        self.job_id += 1
        job = CmdJob(job_id, cmd, callback)
        self.jobs[job_id] = job
        job.start()
        return job_id

    def wait_job(self, job_id):
        job = self.jobs[job_id]
        job.join()
        del(self.jobs[job_id])

    def close(self):
        # FIXME: should we kill jobs still running?
        for job in self.jobs.values():
            job.join()


#------------------------------------------------------------------------------

if __name__=='__main__':
    def job_callback(job_id, type, data):
        print "job_id=%s type=%s, data=%s" % (job_id, type, data)

    job_pool = CmdJobPool()

    cmd = '/bin/cat ./cmd_job.py'
    job_id = job_pool.new_job(cmd, job_callback)

    job_pool.close()
