# /usr/bin/python -E

#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

__all__ = ['config_init',
           'cfg',
           'LOG_CATEGORIES',
          ]

import getopt
import sys
import os
import ConfigParser
import textwrap

CFG_FILE = os.path.join('/etc/setroubleshoot', "%s.cfg" % 'setroubleshoot')

LOG_CATEGORIES = ['rpc','xml','cfg','alert','sig','plugin','avc',
	          'email','gui', 'program','database','server','dbus',
                  'stats']

defaults = {
'general' : {
    'pid_file' : {
        'value'       : '/var/run/setroubleshootd.pid',
        'description' : '',
        'readOnly'    : False,
    },
    'pkg_name' : {
        'value'       : 'setroubleshoot',
        'description' : '',
        'readOnly'    : True,
    },
    'data_dir' : {
        'value'       : '/usr/share/setroubleshoot',
        'description' : '',
        'readOnly'    : True,
    },
    'config_dir' : {
        'value'       : '/etc/setroubleshoot',
        'description' : '',
        'readOnly'    : True,
    },
},
'audit' : {
    'socket_path' : {
        'value'       : '/var/run/audit_events',
        'description' : 'unix domain socket used to listen for audit messages',
    },
    'retry_interval' : {
        'value'       : 60,
        'description' : 'number of seconds to wait before trying to connect to audit socket again in the event of socket failure',
    },
},
'alert' : {
    'glade_file' : {
        'value'       : os.path.join('/usr/share/setroubleshoot/gui','setroubleshoot.glade'),
        'description' : '',
    },
    'status_popup_menu' : {
        'value'       : os.path.join('/usr/share/setroubleshoot/gui','status_popup_menu.glade'),
        'description' : '',
    },
    'status_icon' : {
        'value'       : os.path.join('/usr/share/setroubleshoot/gui','setroubleshoot_icon.png'),
        'description' : '',
    },
    'browser_glade_file' : {
        'value'       : os.path.join('/usr/share/setroubleshoot/gui','setroubleshoot_browser.glade'),
        'description' : '',
    },
    'use_notification' : {
        'value'       : 'True',
        'description' : 'use balloon notification in addition to status icon',
    },
},
'plugins' : {
    'plugin_dir' : {
        'value'       : '/usr/share/setroubleshoot/plugins',
        'description' : '',
    },
},
'session_dbus' : {
    'bus_name' : {
        'value'       : 'com.redhat.setroubleshoot',
        'description' : '',
    },
    'object_path' : {
        'value'       : '/com/redhat/setroubleshoot/alert_object',
        'description' : '',
    },
    'interface' : {
        'value'       : 'com.redhat.setroubleshoot.alert_iface',
        'description' : '',
    },
    'service' : {
        'value'       : 'com.redhat.setroubleshoot',
        'description' : '',
    },
},
'database' : {
    'database_dir' : {
        'value'       : '/var/lib/setroubleshoot',
        'description' : '',
    },
    'filename' : {
        'value'       : 'audit_listener',
        'description' : '',
    },
},
'local_fault_server' : {
    'socket_type' : {
        'value'       : 'unix',
        'description' : '',
    },
    'path' : {
        'value'       : os.path.join('/var/run/setroubleshoot','setroubleshoot_server'),
        'description' : '',
        'readOnly'    : False,
    },
    'host' : {
        'value'       : 'localhost',
        'description' : '',
    },
    'port' : {
        'value'       : '69783',     # FIXME: figure out defined port,
        'description' : '',
    },
},
'local_http_server' : {
    'docroot' : {
        'value'       : '@pkgwebdir@',
        'description' : '',
        'readOnly'    : True,
    },
    'socket_type' : {
        'value'       : 'inet',
        'description' : '',
    },
    'path' : {
        'value'       : os.path.join('/var/run/setroubleshoot','setroubleshoot_httpd'),
        'description' : '',
        'readOnly'    : False,
    },
    'host' : {
        'value'       : 'localhost',
        'description' : '',
    },
    'port' : {
        'value'       : '3267',     # FIXME: figure out defined port,
        'description' : '',
    },
},
'socket' : {
    'buf_size' : {
        'value'       : '2048',
        'description' : '',
    },
    'timeout' : {
        'value'       : '5',
        'description' : '',
    },
},
'fix_command' : {
    'prog_search_path' : {
        'value'       : '/usr/bin:/usr/sbin:/sbin',
        'description' : '',
    },
},
'setroubleshootd_log' : {
    'level' : {
        'value'       : 'warning',
        'description' : '''
Global logging level. Levels are the same as in the python logging
module, but are case insenstive. The level may also be specified as an
integer. The defined levels in severity order are:[CRITICAL, ERROR,
WARNING, INFO, DEBUG]''',
    },
    'categories' : {
        'value'       : '',
        'description' : '''
Comma seperated list of logging categories. Each categories may
optionally be followed by a colon(:) and a logging level, which either
may be one of the symbolic levels or numeric, e.g. ipc:info. If no
level is defined the default level is used. If the category name is
preceded by a tilde (~) the category is not logged at all.
The list of available categories is: [%s]''' % ', '.join(LOG_CATEGORIES),
    },
    'filename' : {
        'value'       : '/var/log/setroubleshoot/setroubleshootd.log',
        'description' : '',
    },
    'filemode' : {
        'value'       : 'w',
        'description' : 'should be "w" or "a" for write or append respectively'
    },
    'format' : {
        'value'       : '%(asctime)s [%(name)s.%(levelname)s] %(message)s',
        'description' : '',
    },
    'console' : {
        'value'       : 'False',
        'description' : 'True|False, also log to the console',
    },
    'profile' : {
        'value'       : 'False',
        'description' : 'True|False, gather statistics',
    },
},
'sealert_log' : {
    'level' : {
        'value'       : 'warning',
        'description' : '''
Global logging level. Levels are the same as in the python logging
module, but are case insenstive. The level may also be specified as an
integer. The defined levels in severity order are:[CRITICAL, ERROR,
WARNING, INFO, DEBUG]''',
    },
    'categories' : {
        'value'       : '',
        'description' : '''
Comma seperated list of logging categories. Each categories may
optionally be followed by a colon(:) and a logging level, which either
may be one of the symbolic levels or numeric, e.g. ipc:info. If no
level is defined the default level is used. If the category name is
preceded by a tilde (~) the category is not logged at all.
The list of available categories is: [%s]''' % ', '.join(LOG_CATEGORIES),
    },
    'filename' : {
        'value'       : '/tmp/sealert.log',
        'description' : ''
    },
    'filemode' : {
        'value'       : 'w',
        'description' : 'should be "w" or "a" for write or append respectively'
    },
    'format' : {
        'value'       : '%(asctime)s [%(name)s.%(levelname)s] %(message)s',
        'description' : '',
    },
    'console' : {
        'value'       : 'False',
        'description' : 'True|False, also log to the console',
    },
    'profile' : {
        'value'       : 'False',
        'description' : 'True|False, gather statistics',
    },
},
'access' : {
    'client_users' : {
	'value'	      : '*',
	'description' : '''
Comma-separated list of users allowed to run the client and connect to
the local fault server and therefore see security denials.
Also accepts '*' to allow all users to connect.'''
    },
    'fix_cmd_users' : {
	'value'	      : 'root',
	'description' : '''
Comma-separated list of users allowed to run the fix commands with
root privileges. Members of this list can execute the fix commands
specified in any alert. The command is executed with root privileges
so you should be very caeful who you add to this list as you are
granting them significant power to alter the security settings of this
system. The wildcard '*' is NOT allowed.'''

    },
},
'email' : {
    'smtp_host' : {
        'value'       : 'localhost',
        'description' : 'The SMTP server address',
    },
    'smtp_port' : {
        'value'       : '25',
        'description' : 'The SMTP server port',
    },
    'from_address' : {
        'value'       : 'SELinux_Troubleshoot',
        'description' : 'The From: email header',
    },
    'subject' : {
        'value'       : 'SELinux AVC Alert',
        'description' : 'The Subject: email header',
    },
    'recipients' : {
        'value'       : '',
        'description' : 'Comma-seperated list of email addresses alerts will be sent to'
    },
},
}

def config_init():
    global cfg
    cfg = read_configuration(defaults)

def read_configuration(defaults):
    cfg = ConfigParser.SafeConfigParser()
    cfg.read(CFG_FILE)

    default_sections = defaults.keys()
    for default_section in default_sections:
        if not cfg.has_section(default_section):
            cfg.add_section(default_section)
        for default_option,properties in defaults[default_section].items():
            value    = properties['value']
            readOnly = properties.get('readOnly', False)
            if not cfg.has_option(default_section, default_option):
                cfg.set(default_section,default_option,value)
            else:
                if readOnly:
                    # FIXME: loggers have not been initialized yet
                    #log_cfg.warning("[%s] %s cannot be set in config file", default_section, default_option)
                    cfg.set(default_section, default_option, value)
                    
    return cfg

def dump_defaults(defaults, showReadOnly=False):
    wrap = textwrap.TextWrapper(width=78,
                                initial_indent='# ', subsequent_indent='# ')

    sections = defaults.keys()
    sections.sort()
    for section in sections:
        visibleOptions = 0
        for option,properties in defaults[section].items():
            readOnly    = properties.get('readOnly', False)
            if showReadOnly or not readOnly:
                visibleOptions += 1
        if visibleOptions > 0:
            print "[%s]" % section
            for option,properties in defaults[section].items():
                value       = properties['value']
                readOnly    = properties.get('readOnly', False)
                description = properties.get('description', '')

                if readOnly and not showReadOnly:
                    continue

                if not description:
                    description = 'No Description Available'
                print wrap.fill('%s: ' % option + description)
                if readOnly:
                    print '# READ ONLY, default = "%s"' % (value)
                else:
                    print "%s = %s" % (option, value)
                print

def dump_configuration(cfg):
    sections = cfg.sections()
    sections.sort()
    for section in sections:
        options = cfg.options(section)
        options.sort()
        for option in options:
            value = cfg.get(section, option)
            print "[%s] %s = %s" % (section, option, value)
        print


# -----------------------------------------------------------------------------

if __name__ == '__main__':

    def usage():
	print '''
    -d generate default config file
    -h help
    '''
    try:
	opts, args = getopt.getopt(sys.argv[1:], "dh", ["defaults", "help"])
    except getopt.GetoptError:
	# print help information and exit:
	usage()
	sys.exit(2)

    do_dump_defaults = False
    for o, a in opts:
	if o in ("-d", "--defaults"):
            do_dump_defaults = True

	if o in ("-h", "--help"):
	    usage()
	    sys.exit()

    if do_dump_defaults:
        dump_defaults(defaults)
