#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

# Needed to silence warnings if X display is not present
import warnings
warnings.filterwarnings('ignore', 'could not open display')

from rhpl.translate import _
import pygtk
pygtk.require("2.0")
import gtk, gtk.glade
import gtkhtml2
import pynotify
import gobject 
import gnome
import os
import datetime
import selinux

from setroubleshoot.config import cfg
from setroubleshoot.errcode import *
from setroubleshoot.signature import *
from setroubleshoot.util import *
from setroubleshoot.log import *

#------------------------------------------------------------------------------

__all__ = ['map_column_types',
           'ColumnAttr',
           'Applet',
           'DateTimeDialog',
           'FileChooserDialog',
           'get_display',
          ]


#------------------------------------------------------------------------------

map_column_types = {
    'icon'     : gtk.gdk.Pixbuf,
    'string'   : gobject.TYPE_STRING,
    'int'      : gobject.TYPE_INT,
    'pyobject' : gobject.TYPE_PYOBJECT,
    'toggle'   : gobject.TYPE_BOOLEAN
    }

#------------------------------------------------------------------------------

def get_display():
    try:
        dpy = gtk.gdk.Display('')
        dpy_name = dpy.get_name()
        return dpy_name
    except RuntimeError, e:
        return None

#------------------------------------------------------------------------------

class ColumnAttr:
    def __init__(self, column, title, visible, type, sort_id=None, cell_data_func=None, edit_callback_func=None):
        self.column = column
        self.title = title
        self.visible = visible
        self.type = type
        self.sort_id = sort_id
        self.cell_data_func = cell_data_func
        self.edit_callback_func = edit_callback_func

#------------------------------------------------------------------------------

class Applet:

    def get_widget(self, name):
        widget_name = '%s_widget' % name
        widget = self.wTree.get_widget(name)
        if widget is None:
            raise ProgramError(ERR_CANNOT_CREATE_GUI, "cannot find %s widget, GUI fails" % name)
        
        self.__dict__[widget_name] = widget
        return widget

    def init_list_view(self, name, column_attrs=None, rows=None):
        if column_attrs is None:
            column_attrs = [ColumnAttr(0, 'default', True, gobject.TYPE_STRING)]

        column_attrs.sort(lambda a,b: cmp(a.column, b.column))
        column_types = [map_column_types[attr.type] for attr in column_attrs]


        widget = self.get_widget(name)

        model_name  = '%s_model'  % name
        model = gtk.ListStore(*column_types)
        self.__dict__[model_name]  = model

        widget.set_model(model)

        for column_attr in column_attrs:
            if column_attr.type == 'string' or column_attr.type == 'int':
                cell = gtk.CellRendererText()
                tv_column = gtk.TreeViewColumn(column_attr.title, cell, text=column_attr.column)
                tv_column.set_visible(column_attr.visible)
            elif column_attr.type == 'icon':
                cell = gtk.CellRendererPixbuf()
                tv_column = gtk.TreeViewColumn(column_attr.title, cell, pixbuf=column_attr.column)
                tv_column.set_visible(column_attr.visible)
            elif column_attr.type == 'toggle':
                cell = gtk.CellRendererToggle()
                cell.set_property('activatable', True)
                cell.connect( 'toggled', column_attr.edit_callback_func, model)
                tv_column = gtk.TreeViewColumn(column_attr.title, cell)
                tv_column.set_visible(column_attr.visible)
            elif column_attr.type == 'pyobject':
                cell = gtk.CellRendererText()
                tv_column = gtk.TreeViewColumn(column_attr.title, cell)
                tv_column.set_visible(column_attr.visible)
            else:
                raise ValueError('unknown column type = %s' % column_attr.type)
            if column_attr.cell_data_func:
                tv_column.set_cell_data_func(cell, column_attr.cell_data_func, column_attr.column)
            if column_attr.sort_id is not None:
                tv_column.set_sort_column_id(column_attr.sort_id)
                tv_column.set_sort_indicator(False)
            widget.append_column(tv_column)
        
        if rows is not None:
            for row in rows:
                model.append(row)


    def init_text_view(self, name, text=None):
        widget = self.get_widget(name)

        text_buf_name  = '%s_text_buf'  % name
        text_buf = widget.get_buffer()
        self.__dict__[text_buf_name]  = text_buf

        if text is not None:
            text_buf.set_text(text)

    def init_text_entry(self, name, text=None):
        widget = self.get_widget(name)
        
        if text is not None:
            widget.set_text(text)

    def init_combo_box(self, name, column_attrs=None, rows=None):
        if column_attrs is None:
            column_attrs = [ColumnAttr(0, 'default', True, gobject.TYPE_STRING)]

        column_attrs.sort(lambda a,b: cmp(a.column, b.column))
        column_types = [map_column_types[attr.type] for attr in column_attrs]


        widget = self.get_widget(name)

        model_name  = '%s_model'  % name
        model = gtk.ListStore(*column_types)
        self.__dict__[model_name]  = model

        widget.set_model(model)
        widget.clear()

        column_index = 0
        for column_attr in column_attrs:
            if column_attr.visible:
                if column_attr.type == 'string' or column_attr.type == 'int':
                    cell = gtk.CellRendererText()
                    widget.pack_end(cell, True)
                    widget.add_attribute(cell, 'text', column_index)
                elif column_attr.type == 'icon':
                    cell = gtk.CellRendererPixbuf()
                    widget.pack_end(cell, True)
                    widget.add_attribute(cell, 'pixbuf', column_index)
                elif column_attr.type == 'pyobject':
                    cell = gtk.CellRendererText()
                    widget.pack_end(cell, True)
                    widget.add_attribute(cell, 'text', column_index)
                else:
                    raise ValueError('unknown column type = %s' % column_attr.type)
            column_index += 1
        
        
        if rows is not None:
            for row in rows:
                model.append(row)

    def create_htmlview(self, container):
        view = gtkhtml2.View()
        doc = gtkhtml2.Document()
        container.set_hadjustment(view.get_hadjustment())
        container.set_vadjustment(view.get_vadjustment())
        view.set_document(doc)
        container.add(view)

        return (view, doc)

#------------------------------------------------------------------------------

class DateTimeDialog:
    AM = 0
    PM = 1

    def __init__(self):
        program_name = 'setroubleshoot browser'
        program_version = '1.0'
        gnome.init(program_name, program_version)
        self.window_name = "browser"
        self.window_name = "date_time_dialog"
        glade_file = '../gui/setroubleshoot_browser.glade'
        self.wTree = gtk.glade.XML(glade_file, self.window_name)

        self.dialog = self.wTree.get_widget("date_time_dialog")

        self.dt = datetime.datetime.now()

        self.calendar = self.wTree.get_widget("calendar")
        self.hour_spinbutton = self.wTree.get_widget("hour_spinbutton")
        self.minute_spinbutton = self.wTree.get_widget("minute_spinbutton")
        self.am_pm_combobox = self.wTree.get_widget("am_pm_combobox")
        self.cur_datetime_echo = self.wTree.get_widget("cur_datetime_echo")

        self.calendar.connect("month_changed", self.on_update)
        self.calendar.connect("day_selected", self.on_update)
        self.calendar.connect("day_selected_double_click", self.on_update)

        self.hour_spinbutton.connect("value_changed", self.on_update)
        self.minute_spinbutton.connect("value_changed", self.on_update)
        self.am_pm_combobox.connect("changed", self.on_update)

    def on_update(self, widget):
        self.echo_cur_datetime()

    def echo_cur_datetime(self):
        self.get_datetime_from_widgets()
        self.cur_datetime_echo.set_text(self.datetime_echo_string())

    def set_time(self, hour, minute, am_pm):
        date = self.dt.date()
        time = datetime.time(hour, minute)
        self.dt = self.dt.combine(date, time)
        self.update_view()
        return self.dt

    def update_view(self):
        hour = self.dt.hour
        if hour <= 12:
            am_pm = self.AM
        else:
            hour -= 12
            am_pm = self.PM
        minute = self.dt.minute

        year = self.dt.year
        month = self.dt.month - 1
        day = self.dt.day

        self.calendar.select_month(month, year)
        self.calendar.select_day(day)

        self.hour_spinbutton.set_value(hour)
        self.minute_spinbutton.set_value(minute)
        self.am_pm_combobox.set_active(am_pm)
        self.echo_cur_datetime()

    def datetime_echo_string(self):
        if self.am_pm_combobox.get_active() == self.AM:
            am_pm = ' AM'
        else:
            am_pm = ' PM'
        str = self.dt.strftime('%a %b %e %Y %l:%M') + am_pm
        return str

    def get_datetime_from_widgets(self):
        hour = self.hour_spinbutton.get_value_as_int()
        minute = self.minute_spinbutton.get_value_as_int()
        am_pm = self.am_pm_combobox.get_active()
        year, month, day = self.calendar.get_date()
        #print "year=%s month=%s day=%s hour=%s minute=%s" % (year, month, day, hour, minute)
        # calendar widget 1 based, datetime object 0 based
        month += 1
        self.dt = datetime.datetime(year, month, day, hour, minute)
        # adjust for am/pm
        if am_pm == self.PM:
            # pretend 12am == 12pm on the same day
            if hour != 12:
                self.dt += datetime.timedelta(hours=12)
        return self.dt

    def get_datetime(self):
        return self.get_datetime_from_widgets()

    def dialog_get_datetime(self):
        result = None
        self.set_time(12, 0, self.PM)
	if self.dialog.run() == gtk.RESPONSE_OK:
            result = self.get_datetime_from_widgets()
	self.dialog.destroy()
        return result


#------------------------------------------------------------------------------
class FileChooserDialog:
    def __init__(self):
        pass
    
    def get_filename(self, title):
	result = None
	file_open = gtk.FileChooserDialog(title, action=gtk.FILE_CHOOSER_ACTION_OPEN,
            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))

	if file_open.run() == gtk.RESPONSE_OK:
		result = file_open.get_filename()
	file_open.destroy()
        return result
        
#------------------------------------------------------------------------------
