#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

__all__ = ["RunCmdGUI",
          ]

import popen2, os, signal, shlex, fcntl

from setroubleshoot.config import cfg

def WaitStatus(status):
    exitStatus = (status >> 8) & 0xFF
    signal     = status & 0xFF
    return (exitStatus, signal)


import pygtk
pygtk.require("2.0")
import gtk, gtk.glade
import gobject

STATE_NOT_RUN = 1
STATE_RUNNING = 2
STATE_DONE    = 3


class RunCmdGUI(object):
    def __init__(self, cmd=""):
        self.cmd = cmd
        self.pid = None
        self.exitStatus = 0
        self.exitSignal = 0
        self.windowTitle = "SELinux Fix"
        windowName = "cmd_win"
        self.wTree = gtk.glade.XML(cfg.get('alert','glade_file'), windowName)
        dic = {"on_cmd_run_button_clicked"    : self.onRun,
               "on_cmd_stop_button_clicked"   : self.onCancel,
               "on_cmd_ok_button_clicked"     : self.onOK,
               "on_cmd_win_delete_event"      : self.onWinClose,
               "on_run_command_activate"      : self.onRun,
               } 
        self.wTree.signal_autoconnect(dic) 

        self.window_widget     = self.wTree.get_widget("cmd_win")
        self.cmd_widget        = self.wTree.get_widget("run_command")
        self.cmd_stdout_widget = self.wTree.get_widget("cmd_stdout")
        self.cmd_stderr_widget = self.wTree.get_widget("cmd_stderr")
        self.cmd_status_widget = self.wTree.get_widget("cmd_status")
        self.cmd_run_button    = self.wTree.get_widget("cmd_run_button")
        self.cmd_stop_button   = self.wTree.get_widget("cmd_stop_button")
        self.cmd_ok_button     = self.wTree.get_widget("cmd_ok_button")

        self.SetState(STATE_NOT_RUN)

    def onWinClose(self, widget, event, data=None):
        if __name__ == "__main__":
            gtk.main_quit()

    def onCancel(self, widget, data=None):
        if self.pid:
            os.kill(self.pid, signal.SIGKILL)
        #self.SetState(STATE_DONE)

    def onOK(self, widget, data=None):
        if __name__ == "__main__":
            gtk.main_quit()
        self.window_widget.destroy()

    def SetState(self, state):
        self.state = state

        if self.state == STATE_NOT_RUN:
            self.statusMsg = "Not Run Yet"
            self.cmd_run_button.set_sensitive(True)
            self.cmd_stop_button.set_sensitive(False)
            self.cmd_ok_button.set_sensitive(False)
        elif self.state == STATE_RUNNING:
            self.statusMsg = "Running..."
            self.window_widget.set_title("Run: " + self.windowTitle)
            self.cmd_run_button.set_sensitive(False)
            self.cmd_stop_button.set_sensitive(True)
            self.cmd_ok_button.set_sensitive(False)
        elif self.state == STATE_DONE:
            if self.exitSignal != 0:
                self.statusMsg = "Interrupted, exit on signal %d" % self.exitSignal
            elif self.exitStatus == 0:
                self.statusMsg = "Success"
            else:
                self.statusMsg = "Failed, exit status = %d" % self.exitStatus

            self.window_widget.set_title("Done: " + self.windowTitle)
            self.cmd_run_button.set_sensitive(False)
            self.cmd_stop_button.set_sensitive(False)
            self.cmd_ok_button.set_sensitive(True)
        else:
            raise ValueError("unknown run state = %s" % self.state)
        
        self.cmd_status_widget.set_text(self.statusMsg)


    def onRun(self, widget, data=None):
        self.RunCmd()

    def onIORead(self, fd, condition, text_view):
        if condition & gobject.IO_IN:
            data = os.read(fd, 1024)
            if data:
                text_buf = text_view.get_buffer()
                end_iter = text_buf.get_end_iter()
                text_buf.insert(end_iter, data)
                text_view.scroll_to_mark(text_buf.get_insert(), 0)
        if condition & gobject.IO_HUP:
            return False
        else:
            return True

    def onChildExit(self, pid, condition):
        self.exitStatus, self.exitSignal = WaitStatus(condition)
        self.SetState(STATE_DONE)

    def RunCmd(self):
        self.SetState(STATE_RUNNING)
        self.cmd = self.cmd_widget.get_text()
        self.pid, self.stdin, self.stdout, self.stderr = \
            gobject.spawn_async(shlex.split(self.cmd),
            standard_output=True, standard_error=True,
            flags=gobject.SPAWN_DO_NOT_REAP_CHILD | gobject.SPAWN_SEARCH_PATH)
        gobject.child_watch_add(self.pid, self.onChildExit)
        gobject.io_add_watch(self.stdout, gobject.IO_IN, self.onIORead,
                             self.cmd_stdout_widget)
        gobject.io_add_watch(self.stderr, gobject.IO_IN, self.onIORead,
                             self.cmd_stderr_widget)

    def ShowCmd(self, cmd=None):
        if cmd is not None:
            self.cmd = cmd
        self.cmd_widget.set_text(self.cmd)
        self.cmd_stdout_widget.get_buffer().set_text("")
        self.cmd_stderr_widget.get_buffer().set_text("")

        self.SetState(STATE_NOT_RUN)


if __name__ == "__main__":

    cmdWin = RunCmdGUI()
    cmdWin.ShowCmd("/bin/cat ./runcmd.py")
    gtk.main()
