
#
# Authors: Karl MacMillan <kmacmillan@mentalrootkit.com>
#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from setroubleshoot.util import *
from setroubleshoot.Plugin import Plugin
from rhpl.translate import _
import selinux

class plugin(Plugin):
    summary = _('''
    SELinux is preventing $SOURCE_PATH ($SOURCE_TYPE) "$ACCESS" access to $TARGET_PATH ($TARGET_TYPE).
    ''')

    problem_description = _('''

    SELinux denied access requested by $SOURCE_PATH. It is not
    expected that this access is required by $SOURCE_PATH and this access
    may signal an intrusion attempt. It is also possible that the specific
    version or configuration of the application is causing it to require
    additional access. Please file a <a
    href="http://bugzilla.redhat.com/bugzilla/enter_bug.cgi">bug
    report</a> against this package.

    ''')

    fix_description = _('''
    Sometimes labeling problems can cause SELinux denials.  You could try to
    restore the default system file context for $TARGET_PATH,

    restorecon -v $TARGET_PATH.

    There is currently no automatic way to allow this access. Instead, you
    can generate a local policy module to allow this access - see
    <a href="http://fedora.redhat.com/docs/selinux-faq-fc5/#id2961385">FAQ</a> - or you can
    disable SELinux protection entirely for the application. Disabling
    SELinux protection is not recommended.
    Please file a <a href="http://bugzilla.redhat.com/bugzilla/enter_bug.cgi">bug report</a>
    against this package.

    Changing the "$BOOLEAN" boolean to true will disable SELinux protection
    this application: "setsebool -P $BOOLEAN=1."
    ''')

    fix_cmd = 'setsebool -P $BOOLEAN=1'

    def __init__(self):
        Plugin.__init__(self, __name__)
        self.set_priority(75)

    def analyze(self, avcinfo):
        bool_name=avcinfo.avc.scontext.type[:-2]+"_disable_trans"
        if selinux.security_get_boolean_active(bool_name) >= 0:
            avcinfo.set_boolean(bool_name)
            return self.report(avcinfo, None, self.summary, self.problem_description,
                               self.fix_description, self.fix_cmd)
        return None
