#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from setroubleshoot.util import *
from setroubleshoot.Plugin import Plugin
from rhpl.translate import _
import re


class plugin(Plugin):
    summary =_('''
    SELinux is preventing the $SOURCE_PATH from using potentially mislabeled files $TARGET_PATH ($TARGET_TYPE).
    ''')

    problem_description = _('''
    SELinux has denied the $SOURCE_PATH access to potentially
    mislabeled files $TARGET_PATH.  This means that SELinux will not
    allow http to use these files.  Many third party apps install html files
    in directories that SELinux policy can not predict.  These directories
    have to be labeled with a file context which httpd can accesss.
    ''')

    fix_description = _('''
    If you want to change the file context of $TARGET_PATH so that the httpd
    daemon can access it, you need to execute it using
    chcon -t httpd_sys_content_t.$TARGET_PATH.  You can
    look at the httpd_selinux man page for additional information.
    ''')

    def __init__(self):
        Plugin.__init__(self, __name__)

    def analyze(self, avcinfo):
        if avcinfo.avc.sourceTypeMatch("httpd_t httpd_sys_script_t httpd_user_script_t httpd_staff_script_t") and \
               not avcinfo.avc.targetTypeMatch("user_home_t staff_home_t user_tmp_t staff_tmp_t tmp_t"):
            return self.report(avcinfo, _("File Label"), self.summary, self.problem_description,
                               self.fix_description, self.fix_cmd)
        return None

